#!/usr/bin/python
# -*- coding: utf-8 -*-

from Screens.Screen import Screen
from Screens.MessageBox import MessageBox
from Components.Sources.StaticText import StaticText
from Components.ActionMap import ActionMap
from Components.ConfigList import ConfigListScreen, ConfigList
from Components.config import config, getConfigListEntry
from enigma import getDesktop
import json
import websocket
import ssl
import base64
from os import path as os_path
#import time

plugin_version = "0.1"
tv_connection = None
sz_w = getDesktop(0).size().width()

class SamsungVolumeRemoteSetup(Screen, ConfigListScreen):
	if sz_w == 1920:	#1920 x 1080 FHD
		skin = """
		<screen name="SamsungVolumeRemoteSetup" position="center,300" size="1220,480">
			<ePixmap pixmap="skin_default/buttons/red.png" position="10,0" size="300,70" scale="stretch" alphatest="on" />
			<ePixmap pixmap="skin_default/buttons/green.png" position="310,0" size="300,70" scale="stretch" alphatest="on" />
			<ePixmap pixmap="skin_default/buttons/yellow.png" position="610,0" size="300,70" scale="stretch" alphatest="on" />
			<ePixmap pixmap="skin_default/buttons/blue.png" position="910,0" size="300,70" scale="stretch" alphatest="on" />
			<widget source="key_red" render="Label" position="10,0" zPosition="1" size="300,70" font="Regular;32" halign="center" valign="center" backgroundColor="#9f1313" transparent="1" />
			<widget source="key_green" render="Label" position="310,0" zPosition="1" size="300,70" font="Regular;32" halign="center" valign="center" backgroundColor="#1f771f" transparent="1" />
			<widget source="key_yellow" render="Label" position="610,0" zPosition="1" size="300,70" font="Regular;32" halign="center" valign="center" backgroundColor="#a08500" transparent="1" />
			<widget source="key_blue" render="Label" position="910,0" zPosition="1" size="300,70" font="Regular;32" halign="center" valign="center" backgroundColor="#18188b" transparent="1" />
			<ePixmap pixmap="skin_default/div-h.png" position="0,80" zPosition="2" size="1220,2" />
			<widget name="config" enableWrapAround="1" position="20,95" scrollbarMode="showOnDemand" size="1180,320" />
		</screen>"""
	else:
		# HD 1280720
		skin = """
		<screen name="SamsungVolumeRemoteSetup" position="center,200" size="800,250">
			<ePixmap pixmap="skin_default/buttons/red.png" position="0,0" size="200,40" scale="stretch" alphatest="on" />
			<ePixmap pixmap="skin_default/buttons/green.png" position="200,0" size="200,40" scale="stretch" alphatest="on" />
			<ePixmap pixmap="skin_default/buttons/yellow.png" position="400,0" size="200,40" scale="stretch" alphatest="on" />
			<ePixmap pixmap="skin_default/buttons/blue.png" position="600,0" size="200,40" scale="stretch" alphatest="on" />
			<widget source="key_red" render="Label" position="0,0" zPosition="1" size="200,40" font="Regular;22" halign="center" valign="center" backgroundColor="#9f1313" transparent="1" />
			<widget source="key_green" render="Label" position="200,0" zPosition="1" size="200,40" font="Regular;22" halign="center" valign="center" backgroundColor="#1f771f" transparent="1" />
			<widget source="key_yellow" render="Label" position="400,0" zPosition="1" size="200,40" font="Regular;22" halign="center" valign="center" backgroundColor="#a08500" transparent="1" />
			<widget source="key_blue" render="Label" position="600,0" zPosition="1" size="200,40" font="Regular;22" halign="center" valign="center" backgroundColor="#18188b" transparent="1" />
			<ePixmap pixmap="skin_default/div-h.png" position="0,55" zPosition="2" size="800,2" />
			<widget name="config" enableWrapAround="1" position="10,70" size="780,130" scrollbarMode="showOnDemand" />
		</screen>"""
		
	def __init__(self, session):
		Screen.__init__(self, session)
		
		self.setTitle("SamsungVolumeRemote Setup - v%s" % plugin_version)
		
		# Initialize widgets
		self["key_red"] = StaticText(_("Close"))
		self["key_green"] = StaticText(_("Save"))
		self["key_yellow"] = StaticText("(re)connect TV")
		self["key_blue"] = StaticText("About")
		
		# Define Actions
		self["actions"] = ActionMap(["SetupActions", "ColorActions"],
			{
				"cancel": 	self.keyCancel,
				"green": 	self.keySave,
				"yellow": 	self.keyYellow,
				"blue":		self.keyBlue,
			},-2)
		
		self.list = []
		ConfigListScreen.__init__(self, self.list, session = self.session, on_change = self.changedEntry)
		self.createSetup()

	def changedEntry(self):
		cur = self["config"].getCurrent()
		if cur and cur[1] == config.plugins.SamsungRemote.enable:
			self.createSetup()

	def createSetup(self):
		self.list = []
		self.list.append(getConfigListEntry(_("enable")+" "+_("Samsung Volume Remote Control"), config.plugins.SamsungRemote.enable))
		if config.plugins.SamsungRemote.enable.value:
			self.list.append(getConfigListEntry(_("Samsung TV")+" "+_("IP"), config.plugins.SamsungRemote.tv_ip))
		self["config"].setList(self.list)

	def keyYellow(self):
		try:
			connectToSamsungTV(self,timeout=30,newConnect=True)

		except:
			import traceback
			traceback.print_exc()

	def keyBlue(self):
		try:
			self.session.open(MessageBox,"Samsung Volume Remote Control by Sven H",MessageBox.TYPE_INFO,timeout=4)
			
		except:
			import traceback
			traceback.print_exc()


class ConnectionClosed(Exception):
    """Connection was closed."""
    pass

class UnhandledResponse(Exception):
    """Received unknown response."""
    pass

def read_response(self, newConnect=False):
	
	response = tv_connection.recv()
	response = json.loads(response)
	
	print("[SamsungVolumeRemote] response", response)
	if "data" in response and "token" in response["data"]:
		cur_token = response["data"]["token"]
		#print("=== token", cur_token)
		if config.plugins.SamsungRemote.token.value != cur_token:
			#print("=== new token", cur_token)
			config.plugins.SamsungRemote.token.value = cur_token
			config.plugins.SamsungRemote.token.save()
	
	elif "data" in response and "clients" in response["data"]:
		#print("=== last clientconnect", response["data"]["clients"][-1])
		cur_token = response["data"]["clients"][-1]["attributes"]["token"]
		#print("=== token", cur_token)
		if config.plugins.SamsungRemote.token.value != cur_token:
			#print("=== new token", cur_token)
			config.plugins.SamsungRemote.token.value = cur_token
			config.plugins.SamsungRemote.token.save()

	if response["event"] != "ms.channel.connect":
		self.session.open(MessageBox,"connect error",MessageBox.TYPE_INFO)
		raise UnhandledResponse(response)

	if newConnect:
		self.session.open(MessageBox,"connect successful",MessageBox.TYPE_INFO,timeout=4)
		
	print("[SamsungVolumeRemote] Access granted.")

def serialize_string(string):
	if isinstance(string, str):
		string = str.encode(string)
	return base64.b64encode(string).decode("utf-8")

def connectToSamsungTV(self, timeout=5, newConnect=False):
	global tv_connection
	
	port=8002
	SSL_URL_FORMAT = "wss://{}:{}/api/v2/channels/samsung.remote.control?name={}"
	ip = '%d.%d.%d.%d' % tuple(config.plugins.SamsungRemote.tv_ip.value)
	url = SSL_URL_FORMAT.format(ip, port,serialize_string("Dreambox"))
	if not newConnect:
		url += "&token=" + config.plugins.SamsungRemote.token.value
	#print("=== connect url", url)
	
	try:
		tv_connection = websocket.create_connection(url,timeout, sslopt={"cert_reqs": ssl.CERT_NONE})
		read_response(self, newConnect=newConnect)
		
	except:
		import traceback, sys
		traceback.print_exc()
		self.session.open(MessageBox,"connect error",MessageBox.TYPE_INFO)

def sendKeyToSamsungTV(self, key):
	try:
		"""Send a control command."""
		connectToSamsungTV(self)
		
		if not tv_connection:
			raise ConnectionClosed()

		command = json.dumps({
			"method": "ms.remote.control",
			"params": {
				"Cmd": "Click",
				"DataOfCmd": key,
				"Option": "false",
				"TypeOfRemote": "SendRemoteKey",
			}
		})

		print("[SamsungVolumeRemote] Sending Samsung Remote control command: %s", key)
		tv_connection.send(command)
		#time.sleep(0.2)

	except:
		import traceback
		traceback.print_exc()

from Components.VolumeControl import VolumeControl
if VolumeControl.setVolume.__name__ != "own_VolumeControl_setVolume":
	VolumeControl_setVolume_org = VolumeControl.setVolume
if VolumeControl.volMute.__name__ != "own_VolumeControl_volMute":
	VolumeControl_volMute_org = VolumeControl.volMute

def own_VolumeControl_setVolume(self, direction): 
	#print("=== own VolumeControl setVolume",direction)
	if config.plugins.SamsungRemote.enable.value:
		if direction>0:
			key = "KEY_VOLUP"
		else:
			key = "KEY_VOLDOWN"
		#print("=== sendKey", key)
		sendKeyToSamsungTV(self, key)
	else:
		VolumeControl_setVolume_org(self, direction)

def own_VolumeControl_volMute(self, showMuteSymbol=True, force=False): 
	#print("=== own VolumeControl volMute")
	if config.plugins.SamsungRemote.enable.value:
		key = "KEY_MUTE"
		#print("=== sendKey", key)
		sendKeyToSamsungTV(self, key)
	else:
		VolumeControl_volMute_org(self, showMuteSymbol=True, force=False)
	

VolumeControl.setVolume = own_VolumeControl_setVolume
VolumeControl.volMute = own_VolumeControl_volMute
